/* written by John Doug Reynolds, January 2002 */

#include "rgsenergy.h"
#include <PixelsTable.h>
#include <CalServer.h>

void rgsenergy()
{
   string dsname = stringParameter("ccdset");
   bool withoffset = booleanParameter("withoffset");
   bool withgain = booleanParameter("withgain");
   bool withcti = booleanParameter("withcti");

   if ( withoffset ) {
      string offset = stringParameter("offset");
      calServer.ccf()->registerNewCifEntry( CifEntry(offset) );
   }

   struct DataSet *dataset = dataSetServer->open( dsname, Dal::Modify );

   calServer << *dataset;
   struct EnergyCorrector* calEC = calServer.getAtom(calEC);
   struct CtiCorrector *calCC = withcti ? calServer.getAtom(calCC) : 0;

   enum Xmm::DataMode datamode
      = Xmm::toDataMode( dataset->attribute("DATAMODE")->asString() );
   bool spectroscopy = (datamode == Xmm::DATA_SPECTROSCOPY);

   {
      using namespace rgsPixelsLib;

      struct Table *ptab = dataset->table("PIXELS");
      struct Table *ftab = dataset->table("EXPOSURE");

      if ( !ptab->hasColumn("PI") )
	 ptab->addColumn( "PI", Dal::Real32, "calibrated energy" );

      unsigned flags = PixelsTable::PIX | PixelsTable::PHA | PixelsTable::CPI;
      const PixelsTable pix(ptab,0,flags);

      const DalVecJ frame( ftab->column("FRAME") );
      const DalVecD  time( ftab->column("TIME") );

      // 

      CalReal32Vector calpiV(  pix.calpi.data(), pix.rows );
      CalInt16Vector   rawxV(   pix.rawX.data(), pix.rows );
      CalInt16Vector energyV( pix.energy.data(), pix.rows );
      CalInt8Vector    nodeV(   pix.node.data(), pix.rows );

      CalInt16Vector  rawyV(  pix.rawY.data(), spectroscopy ? pix.rows : 0 );
      CalInt8Vector  shapeV( pix.shape.data(), spectroscopy ? pix.rows : 0 );

      CalReal32Vector  tmpV( 0, pix.rows );
      CalReal64Vector timeV( 0, pix.rows );

      unsigned fItr = 0, pItr = 0;
      for (; fItr < ftab->rows() && pItr < pix.rows; ++fItr )
	 if ( frame[fItr] >= pix.frame[pItr] ) {
	    int sameFrame = pix.frame[pItr];
	    double t = time[ frame[fItr] == sameFrame ? fItr : 0 ];
	    while ( pItr < pix.rows && pix.frame[pItr] == sameFrame )
	       timeV[pItr++] = t;
	 }

      while ( pItr < pix.rows ) timeV[pItr++] = time[0];

      // 

      if ( spectroscopy )
	 calEC->offsetCorrect(timeV,rawxV,rawyV,shapeV,nodeV,energyV,calpiV);
      else
	 calEC->offsetCorrect(timeV,rawxV,nodeV,energyV,calpiV);

      CalReal32Vector* v[2] = { &calpiV, &tmpV };
      unsigned i, c = 0;

      if ( withgain ) {
	 calEC->gainCorrect(timeV,nodeV,*v[c],*v[1-c]);
	 c = 1 - c;
      }

      if ( withcti ) {
	 if ( spectroscopy ) {
	    CalInt16Vector sepV( 0, pix.rows );
	    for ( i = 0; i < pix.rows; ++i ) sepV[i] = 1000; // dummy value
	    calCC->ctiCorrect(timeV,rawxV,rawyV,shapeV,sepV,
			      *v[c],*v[1-c],&nodeV);
	 }
	 else
	    calCC->ctiCorrect(timeV,rawxV,pix.bounds[0].row.min,
			      *v[c],*v[1-c],&nodeV);
	 c = 1 - c;
      }

      if ( c ) for ( i = 0; i < pix.rows; ++i ) pix.calpi[i] = tmpV[i];
   }

   dataSetServer->close( dataset );

   if ( withcti ) calServer.releaseAtom(calCC);
   calServer.releaseAtom(calEC);
}
